<?php

declare(strict_types=1);

final class SharingDeleteController extends AbstractController
{
	protected string $route = '/api/v1/sharing/{sharingId}';

	private int $sharingId;

	public function __construct(string $url)
	{
		parent::__construct($url);

		$this->sharingId = (int)$this->getUrlParamInt('sharingId');
	}

	public function handle(): void
	{
		parent::handle();

		if ($this->response->getStatus() !== ServerStatus::OK) {
			return;
		}

		try {
			$sharing = new Sharing($this->sharingId);
			$sharing->Delete();

			$this->response = new ApiJsonResponse();
			$this->response->setParameter('success', true);
		} catch (Throwable $e) {
			$this->response = new ApiJsonResponse(ServerStatus::BAD_REQUEST);
			$this->response->setParameter('success', false);
			$this->response->setMessage($e->getMessage());
		}
	}
}